@@ -44,9 +44,7 @@ fn path_to(f: &str) -> String {
 
 #[test]
 fn test() {
-	let f = buffered_reader::File::open(path_to("gitlab-ci.scfg")).unwrap();
-	let mut d = simple_config::Deserializer::from(f);
-	let scfg = <GitlabCi as serde::de::Deserialize>::deserialize(&mut d).unwrap();
+	let scfg: GitlabCi = simple_config::from_file(path_to("gitlab-ci.scfg")).unwrap();
 
 	let f = buffered_reader::File::open(path_to("gitlab-ci.yml")).unwrap();
 	let mut yaml: GitlabCi = serde_yaml::from_reader(f).unwrap();
