@@ -1,41 +1,28 @@
 pub(crate) struct KeyDeserializer<'a, R: buffered_reader::BufferedReader<()>>{
 	pub root: &'a mut crate::Deserializer<R>,
-	pub key_len: usize,
 }
 
 impl<'a, R: buffered_reader::BufferedReader<()>> KeyDeserializer<'a, R> {
-	fn peek_key(&self) -> &str {
-		let b = &self.root.input.buffer()[..self.key_len];
-		unsafe { crate::from_utf8_unchecked(b) }
-	}
-
-	fn take_key(self) -> (&'a str, crate::Location) {
-		let location = self.root.location;
-		let b = &self.root.input.consume(self.key_len+1)[..self.key_len];
-		self.root.location.add_columns(self.key_len+1);
-		(unsafe { crate::from_utf8_unchecked(b) }, location)
-	}
-
 	fn take_sint<T: std::convert::TryFrom<i128> + std::ops::Neg>(self) -> Result<T, crate::DeserializeError>
 	where
 		<T as std::convert::TryFrom<i128>>::Error: std::fmt::Debug,
 		<T as std::ops::Neg>::Output: std::convert::Into<T>,
 	{
-		let (s, location) = self.take_key();
-		crate::parse_num(s, location)?.to_sint(s).map_err(|e| e.at(location))
+		let (s, location) = self.root.parse_key()?;
+		crate::parse_num(&s, location)?.to_sint(&s).map_err(|e| e.at(location))
 	}
 
 	fn take_uint<T: std::convert::TryFrom<u128>>(self) -> Result<T, crate::DeserializeError>
 	where
 		<T as std::convert::TryFrom<u128>>::Error: std::fmt::Debug,
 	{
-		let (s, location) = self.take_key();
-		crate::parse_num(s, location)?.to_uint(s).map_err(|e| e.at(location))
+		let (s, location) = self.root.parse_key()?;
+		crate::parse_num(&s, location)?.to_uint(&s).map_err(|e| e.at(location))
 	}
 
 	fn take_float(self) -> Result<f64, crate::DeserializeError> {
-		let (s, location) = self.take_key();
-		crate::parse_num(s, location).map(|n| n.to_f64())
+		let (s, location) = self.root.parse_key()?;
+		crate::parse_num(&s, location).map(|n| n.to_f64())
 	}
 }
 
@@ -47,7 +34,7 @@ impl<'d, 'a, R: buffered_reader::BufferedReader<()>> serde::de::Deserializer<'d>
 	}
 
 	fn deserialize_bool<V: serde::de::Visitor<'d>>(self, visitor: V) -> Result<V::Value, crate::DeserializeError> {
-		let (s, location) = self.take_key();
+		let (s, location) = self.root.parse_key()?;
 		let b = match s.as_ref() {
 			"true" => true,
 			"false" => false,
@@ -108,7 +95,7 @@ impl<'d, 'a, R: buffered_reader::BufferedReader<()>> serde::de::Deserializer<'d>
 	}
 
 	fn deserialize_char<V: serde::de::Visitor<'d>>(self, visitor: V) -> Result<V::Value, crate::DeserializeError> {
-		let (s, location) = self.take_key();
+		let (s, location) = self.root.parse_key()?;
 		if s.len() == 1 {
 			visitor.visit_char(s.chars().next().unwrap())
 		} else {
@@ -119,7 +106,7 @@ impl<'d, 'a, R: buffered_reader::BufferedReader<()>> serde::de::Deserializer<'d>
 	}
 
 	fn deserialize_str<V: serde::de::Visitor<'d>>(self, visitor: V) -> Result<V::Value, crate::DeserializeError> {
-		visitor.visit_str(self.take_key().0)
+		visitor.visit_str(&self.root.parse_key()?.0)
 	}
 
 	fn deserialize_string<V: serde::de::Visitor<'d>>(self, visitor: V) -> Result<V::Value, crate::DeserializeError> {
@@ -137,17 +124,17 @@ impl<'d, 'a, R: buffered_reader::BufferedReader<()>> serde::de::Deserializer<'d>
 	}
 
 	fn deserialize_option<V: serde::de::Visitor<'d>>(self, visitor: V) -> Result<V::Value, crate::DeserializeError> {
-		match self.peek_key() {
-			"none" => {
-				self.take_key();
-				visitor.visit_none()
-			}
-			_ => visitor.visit_some(self),
+		if self.root.input.data(1)?.starts_with(b":") {
+			let (k, _) = self.root.parse_key()?;
+			debug_assert_eq!(k, "");
+			visitor.visit_none()
+		} else {
+			visitor.visit_some(self)
 		}
 	}
 
 	fn deserialize_unit<V: serde::de::Visitor<'d>>(self, visitor: V) -> Result<V::Value, crate::DeserializeError> {
-		self.take_key();
+		self.root.parse_key()?;
 		visitor.visit_unit()
 	}
 
@@ -209,7 +196,7 @@ impl<'d, 'a, R: buffered_reader::BufferedReader<()>> serde::de::Deserializer<'d>
 		variants: &'static [&'static str],
 		visitor: V,
 	) -> Result<V::Value, crate::DeserializeError> {
-		let (s, location) = self.take_key();
+		let (s, location) = self.root.parse_key()?;
 		serde::de::IntoDeserializer::into_deserializer(s)
 			.deserialize_enum(name, variants, visitor)
 			.map_err(|e: crate::DeserializeError| e.with_location(location))
